@@ -91,6 +91,10 @@ class FileIO(object):
     def join(*args) -> str:
         return os.path.normpath(os.path.join(*args))
 
+    @staticmethod
+    def join_cwd(*args) -> str:
+        return os.path.normpath(os.path.join(os.getcwd(), *args))
+
     @staticmethod
     def temp(data: object) -> str:
         """ Write a Data Object to a Temp File
@@ -193,6 +197,18 @@ class FileIO(object):
         FileIO.write_json(data, file_path=file_path)
         raise ValueError(f"Wrote Temp File: {file_path}")
 
+    @staticmethod
+    def write_image(data: object,
+                    file_path: str) -> None:
+        """ Write an Image to File
+
+        Args:
+            data (object): the image
+            file_path (str): the absolute and qualified output file path
+        """
+        with open(file_path, 'wb') as f:
+            f.write(data)
+
     @staticmethod
     def write_json(data: object,
                    file_path: str) -> None:
@@ -278,10 +294,18 @@ class FileIO(object):
 
         Raises:
             ValueError: Invalid Input
+            NotImplementedError: Input is neither a List nor Str
         """
 
-        input_text = '\n'.join(lines)
-        FileIO.write_string(input_text=input_text,
+        def get_input_text() -> str:
+            _type = type(lines)
+            if _type == list:
+                return '\n'.join(lines)
+            if _type == str:
+                return str(lines)
+            raise NotImplementedError(_type)
+
+        FileIO.write_string(input_text=get_input_text(),
                             file_path=file_path,
                             file_encoding=file_encoding)
 
@@ -295,7 +319,7 @@ class FileIO(object):
             file_path (str): the absolute and qualified output file path
         """
         with open(file_path, 'w') as file:
-            documents = yaml_dump(d, file)
+            yaml_dump(d, file)
 
     @staticmethod
     def read_yaml(file_path: str) -> dict:
@@ -359,7 +383,7 @@ class FileIO(object):
                 ['A3.26\t1999\t4\t3']
 
             Notes:
-                each output line is a list with one element that requires independent delimitation            
+                each output line is a list with one element that requires independent delimitation
         """
         with codecs_open(file_path, mode="r", encoding=file_encoding) as f:
             for line in csv_reader(f):
@@ -379,6 +403,34 @@ class FileIO(object):
         """
         return len(FileIO.load_files(directory, extension))
 
+    @staticmethod
+    def delete_files(directory: str,
+                     extension: str,
+                     limit: int = None,
+                     blacklist: list = None,
+                     recursive: bool = False) -> list:
+        """ Delete Files from a directory
+
+        Args:
+            directory (str): the absolute and qualified directory path
+            extension (str): the file extension to delete
+            recursive (bool, optional): deletes files recursively if set to True. Defaults to False.
+            limit (int, optional): The number of files to deletes. Defaults to None.
+            blacklist (list, optional): an list of File Names to exclude. Defaults to None.
+
+        Returns:
+            list: list of files deleted
+        """
+        files = FileIO.load_files(directory=directory,
+                                  extension=extension,
+                                  limit=limit,
+                                  blacklist=blacklist,
+                                  recursive=recursive)
+
+        [os.remove(x) for x in files]
+
+        return files
+
     @staticmethod
     def load_files(directory: str,
                    extension: str,
@@ -580,7 +632,7 @@ class FileIO(object):
         try:
             return yaml_load(StringIO(file_data), Loader)
         except YAMLError:
-            raise ValueError(f"Invalid YAML Data")
+            raise ValueError("Invalid YAML Data")
 
     @staticmethod
     def parse_json(file_data: str) -> dict:
@@ -605,4 +657,4 @@ class FileIO(object):
             return json_loads(file_data)
 
         except JSONDecodeError:
-            raise ValueError(f"Invalid JSON Data")
+            raise ValueError("Invalid JSON Data")
