@@ -3,6 +3,9 @@
 """ Text Utility Methods: Common Functions without Special Libraries """
 
 
+from typing import List
+from typing import Optional
+
 from os import sep
 from statistics import mean
 from unicodedata2 import category
@@ -30,6 +33,206 @@ class TextUtils(object):
     """ Text Utility Methods: Common Functions without Special Libraries """
 
     @staticmethod
+    def remove_duplicated_phrases(text_1: str,
+                                  text_2: str) -> str:
+        """ Remove Duplicated Phrases
+
+        Purpose:
+        -   Any Phrase in Text_2 that exists in Text_1 will cause
+            that phrase to be removed from Text_1
+
+        Args:
+            text_1 (str): the text to modify
+            text_2 (str): the read-only text to use for removal
+
+        Returns:
+            str: the modified text_1 (if applicable)
+        """
+
+        if text_2 in text_1:
+            text_1 = text_1.replace(text_2, '').strip()
+
+        return text_1
+
+    @staticmethod
+    def cartesian(matches: list) -> list:
+        """
+        Purpose:
+            Find a list of candidate token sequences within the normalized_input_text
+        Time per Call:
+            0.5ms < x 1.0ms
+        :param matches:
+            a list of 1..* tokens forming a match pattern
+        :return:
+            a list of possible token sequences
+        """
+        from itertools import product
+
+        cartesian = []
+        for element in product(*matches):
+            cartesian.append(element)
+
+        return cartesian
+
+    @staticmethod
+    def sliding_window(tokens: List[str],
+                       window_size: int) -> Optional[List[str]]:
+        """ Perform Sliding Window (e.g., n-gram) extraction over a list of tokens
+
+        Args:
+            tokens (List[str]): the incoming tokens
+            window_size (int): the window (e.g., n-gram) size
+
+        Returns:
+            Optional[List[str]]: the outgoing tokens (if any)
+        """
+
+        if window_size > len(tokens):
+            return None
+
+        if window_size == len(tokens):
+            return [tokens]
+
+        results = []
+        total_tokens = len(tokens)
+
+        for i in range(total_tokens):
+
+            buffer = []
+
+            x = 0
+            while x < window_size:
+
+                pos = x + i
+                if pos < total_tokens:
+                    buffer.append(tokens[pos])
+
+                x += 1
+
+            if len(buffer) == window_size:
+                results.append(buffer)
+                buffer = []
+
+        return results
+
+    @staticmethod
+    def most_similar_phrase(tokens_1: List[str],
+                            tokens_2: List[str],
+                            window_size: int,
+                            score_threshold: float,
+                            debug: bool = False) -> dict:
+        """ Find the Most Similar Phrase in Tokens-2 relative to Tokens-1
+
+        Implementation Note:
+            How is this different from 'longest-common-phrase'?
+
+            In the example below, there are no common sequences between tokens-1 and tokens-2
+            This function will find "nearly similar" sequences and return the most similar
+
+        Args:
+            tokens_1 (list): the first tokenized list
+                ['where', 'is', 'the', 'library', '?']
+            tokens_2 (list): the second tokenized list
+                ['I', 'understand', 'you', 'want', 'to', 'know', 'where', 'the', 'library', 'is', '.']
+            window_size (int, Optional): n-Gram window size for comparison.
+            score_threshold (float): when threshold is met, return the results (if any)
+            debug (bool, Optional): When True, print results to console. default is False.
+
+        Returns:
+            the most similar span (list) with similarity score
+        """
+
+        tokens_1 = [x.lower().strip() for x in tokens_1]
+        tokens_2 = [x.lower().strip() for x in tokens_2]
+
+        t1 = TextUtils.sliding_window(
+            tokens_1,
+            window_size=window_size)
+
+        t2 = TextUtils.sliding_window(
+            tokens_2,
+            window_size=window_size)
+
+        d_results = {}
+
+        for item_1 in [' '.join(x) for x in t1]:
+            for item_2 in [' '.join(x) for x in t2]:
+
+                if item_1 == item_2:
+                    return {100: {"tokens_1": item_1, "tokens_2": item_2}}
+
+                score = TextUtils.jaccard_similarity(item_1, item_2)
+                if debug:
+                    print(f"({item_1}) vs. ({item_2}) = {score}")
+
+                if score >= score_threshold:
+                    d_results[score] = {"tokens_1": item_1, "tokens_2": item_2}
+
+        return d_results
+
+    @staticmethod
+    def longest_common_phrase(tokens_1: List[str],
+                              tokens_2: List[str]) -> Optional[List[str]]:
+        """ Find the Longest Common Phrase in Tokens-2 relative to Tokens-1
+
+        Args:
+            tokens_1 (list): the first tokenized list
+                ['what', 'is', 'the', 'earliest', 'known', 'age', 'of', 'fossils', '?']
+            tokens_2 (list): the second tokenized list
+                ['the', 'earliest', 'known', 'age', 'of', 'fossils', 'is', '3.7', 'billion', 'years', 'old', '.']
+
+        Returns:
+            Optional[List[str]]: the common span (if any)
+                ['the', 'earliest', 'known', 'age', 'of', 'fossils']
+        """
+
+        text_2 = ' '.join(tokens_2).strip()
+
+        def extract(tokens: list,
+                    gram_size: int) -> set:
+            results = []
+
+            if gram_size == len(tokens):
+                return set([' '.join(x) for x in tokens])
+
+            if gram_size == 1:
+                return set([' '.join(x) for x in tokens])
+
+            x = 0
+            y = x + gram_size
+
+            while y <= len(tokens):
+                results.append(tokens[x: y])
+
+                x += 1
+                y = x + gram_size
+
+            return set([' '.join(x) for x in results])
+
+        max_tokens = min([len(tokens_1), len(tokens_2)])
+
+        while max_tokens > 2:
+
+            extracts_1 = extract(tokens_1, max_tokens)
+            extracts_2 = extract(tokens_2, max_tokens)
+
+            common = extracts_1.intersection(extracts_2)
+            if common:
+
+                common_phrase = list(common)[0]
+                if common_phrase not in text_2.lower():
+                    return None
+
+                x = text_2.lower().index(common_phrase)
+                y = x + len(common_phrase)
+
+                return text_2[x: y].split()
+
+            max_tokens -= 1
+
+        return None
+
+    @ staticmethod
     def is_punctuation(char: str) -> bool:
         """ Checks whether `char` is a punctuation character.
 
@@ -59,7 +262,7 @@ class TextUtils(object):
 
         return False
 
-    @staticmethod
+    @ staticmethod
     def has_punctuation(input_text: str):
         """Checks whether the input text has any punctuation characters."""
 
@@ -69,7 +272,7 @@ class TextUtils(object):
 
         return False
 
-    @staticmethod
+    @ staticmethod
     def remove_punctuation(input_text: str):
         """Checks whether the input text has any punctuation characters."""
 
@@ -80,7 +283,7 @@ class TextUtils(object):
 
         return ''.join(buffer)
 
-    @staticmethod
+    @ staticmethod
     def find_subsumed_tokens(tokens: list) -> list:
         """ Find Subsumed Tokens (if any)
 
@@ -138,8 +341,8 @@ class TextUtils(object):
         """ Choose Random Line from Long Input String
 
         The function will segment the input text and if only one line exists, this will be returned
-        if multiple lines exist, the function will randomly choose a single line, assuming the length 
-            of that line is near the mean 
+        if multiple lines exist, the function will randomly choose a single line, assuming the length
+            of that line is near the mean
 
         Use Case:
 
@@ -181,7 +384,7 @@ class TextUtils(object):
             return lines[0]
         return random_line(lines)
 
-    @staticmethod
+    @ staticmethod
     def jaccard_similarity(x: str, y: str) -> float:
         """ returns the jaccard similarity between two lists
 
@@ -189,15 +392,30 @@ class TextUtils(object):
             x (str): the first string to compare
             y (str): the second string to compare
 
+        Raises:
+            ValueError: final score is less than 0.0
+            ValueError: final score is greater than 0.0
+            ValueError: final score is not of type float
+
         Returns:
-            float: the Jaccard similarity score (0 <= x <= 100)
+            float: the Jaccard similarity score (0.0 <= x <= 1.0)
         """
         intersection_cardinality = len(set.intersection(*[set(x), set(y)]))
         union_cardinality = len(set.union(*[set(x), set(y)]))
-        sim = intersection_cardinality/float(union_cardinality)
-        return round(sim, 3)
+        sim = intersection_cardinality / float(union_cardinality)
+        result = round(sim, 3)
 
-    @staticmethod
+        # this is our contract to the consumer!
+        if result < 0.0:
+            raise ValueError
+        if result > 1.0:
+            raise ValueError
+        if type(result) != float:
+            raise ValueError
+
+        return result
+
+    @ staticmethod
     def split_on_len(input_text: str,
                      threshold: int = 7) -> str or None:
         """Insert New Lines into Label Text
@@ -234,7 +452,7 @@ class TextUtils(object):
 
         return sep.join(master).strip()
 
-    @staticmethod
+    @ staticmethod
     def ends_with_punctuation(input_text: str) -> bool:
         """ Determine if Input Text ends with punctuation
 
@@ -249,7 +467,7 @@ class TextUtils(object):
 
         return input_text[-1] in ['.', '?', '!']  # common ending punctuation
 
-    @staticmethod
+    @ staticmethod
     def remove_ending_punctuation(input_text: str) -> str:
         """ Remove Ending (terminating sequence) Punctuation
 
@@ -267,7 +485,7 @@ class TextUtils(object):
             return input_text[:-1]
         return input_text
 
-    @staticmethod
+    @ staticmethod
     def split_on_punctuation(input_text: str,
                              punkt: list = ['!', '?']) -> list:
         """ Split (segment) a line on common punctuation
@@ -289,7 +507,7 @@ class TextUtils(object):
 
         return lines
 
-    @staticmethod
+    @ staticmethod
     def update_spacing(input_text: str) -> str:
         """ Correct Syntactical Spacing Inconsistencies
 
@@ -309,7 +527,7 @@ class TextUtils(object):
 
         return input_text
 
-    @staticmethod
+    @ staticmethod
     def remove_double_spaces(input_text: str) -> str:
         """ Remove Double Spaces in a String
 
@@ -324,7 +542,7 @@ class TextUtils(object):
 
         return input_text
 
-    @staticmethod
+    @ staticmethod
     def update_csvs(input_text: str) -> str:
         """ Synatically correct Comma Separated values in Natural Language
 
@@ -342,7 +560,7 @@ class TextUtils(object):
         """
         return input_text
 
-    @staticmethod
+    @ staticmethod
     def update_determiners(input_text: str) -> str:
         """ Update Determiners in a text string
 
@@ -385,7 +603,7 @@ class TextUtils(object):
 
         return ' '.join(results).strip()
 
-    @staticmethod
+    @ staticmethod
     def startswith_vowel(input_text: str) -> str:
         """ Determine if Input Text starts with a vowel
 
@@ -402,7 +620,7 @@ class TextUtils(object):
 
         return input_text[0].lower() in vowels
 
-    @staticmethod
+    @ staticmethod
     def lower_case(input_text: str) -> str:
         """ Perform Lower Casing on Input
 
@@ -423,7 +641,7 @@ class TextUtils(object):
 
         return ' '.join(tokens).strip()
 
-    @staticmethod
+    @ staticmethod
     def sentence_case(input_text: str) -> str:
         """ Perform Sentence Casing on Input
 
@@ -458,7 +676,7 @@ class TextUtils(object):
 
         return ' '.join(results).strip()
 
-    @staticmethod
+    @ staticmethod
     def title_case(input_text: str) -> str:
         """ Perform Title Casing on Input
 
